#include "../std_lib_inc.h"
#include "../Util/PositionVector.h"

#ifndef MAZE_H
#define MAZE_H

namespace game
{
  class Player;
  class Entity;

  /// Ein Labyrinth.
  /// Besitzt ein Feld
  class Maze
  {
    // class -> members private by default
  private:
    /// Das Spielfeld
    vector<vector<char>> field;
    /// Die Startposition des Spielers
    PositionVector player_start_position;

  public:
    /// Das Spielfeld
    Maze(const vector<vector<char>> play_field, const vector<int> player_start_position, const vector<Entity> enemies);

    /// Kontrolliere, ob der Spieler stirbt
    /// @param player Der Spieler
    /// @return Ob der Spieler tot ist
    bool was_player_killed_by_ghost(const Player& player) const;

    /// Kontrolliere, ob der Spieler am Ziel ist
    /// @param player Der Spieler
    /// @return Ob der Spieler am Ziel ist
    bool is_player_at_goal(const Player& player) const;

    /// Kontrolliere, ob eine bestimmte Position begehbar ist
    /// @param pos Die Position, die überprüft werden soll
    /// @param player_has_key If the player has at least one key
    /// @return Ob die Position begehbar ist
    bool is_pos_free(const PositionVector& pos, const bool& player_has_key) const;

    /// Zeige das Spielfeld in der Konsole an
    /// @param player Der Spieler
    void render(const Player& player, const vector<Entity> entities) const;

    /// Kriege den Wert einer Position
    /// @param pos Die gewollte Position
    /// @return Der Wert der Position
    char get_field(const PositionVector& pos) const;

    /// Ersetze den Wert von einer Position
    /// @param pos Die Position die ersetzt werden soll
    /// @param target Der Wert, auf den die Position gesetzt werden soll
    void update_field(const PositionVector& pos, const char& target);

    /// Kriege die Startposition des Spielers
    /// @return Die Startposition des Spielers
    PositionVector get_player_start_position() const;

    /// Berrechne den Abstand zwischen zwei Vektoren
    /// @return Der Abstand als Differenzvektor
    PositionVector get_distance_pos1_pos2(const PositionVector& pos1, const PositionVector& pos2);

    int calculate_steps(const PositionVector& target_position, PositionVector position, int steps);
  };
} // game

#endif //MAZE_H
